@@ -13,7 +13,7 @@ use 5.00800;
 use Carp 'croak';
 use Scalar::Util;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(encoded_string build_mt render_mt);
 our %EXPORT_TAGS = (
@@ -176,7 +176,7 @@ sub parse {
 
     # Tokenize
     my $state = 'text';
-    my $multiline_expression = 0;
+    my $push_expr = undef;
     my @lines = split /(\n)/, $tmpl;
     while (@lines) {
         my $line = shift @lines;
@@ -186,28 +186,25 @@ sub parse {
             $newline = 1;
         }
         
-        # Perl line without return value
-        if ($line =~ /^$line_start\s+(.*)$/) {
-            push @{$self->{tree}}, ['code', $1];
-            $multiline_expression = 0;
-            next;
-        }
-
-        # Perl line with return value
-        if ($line =~ /^$line_start$expr_mark\s+(.+)$/) {
-            push @{$self->{tree}}, [
-                'expr', $1,
-                $newline ? ('text', "\n") : (),
-            ];
-            $multiline_expression = 0;
-            next;
-        }
-
-        # Comment line, dummy token needed for line count
-        if ($line =~ /^$line_start$cmnt_mark/) {
-            push @{$self->{tree}}, [];
-            $multiline_expression = 0;
-            next;
+        if ($state eq 'text') {
+            # Perl line without return value
+            if ($line =~ /^$line_start\s+(.*)$/) {
+                push @{$self->{tree}}, ['code', $1];
+                next;
+            }
+            # Perl line with return value
+            if ($line =~ /^$line_start$expr_mark\s+(.+)$/) {
+                push @{$self->{tree}}, [
+                    'expr', $1,
+                    $newline ? ('text', "\n") : (),
+                ];
+                next;
+            }
+            # Comment line, dummy token needed for line count
+            if ($line =~ /^$line_start$cmnt_mark/) {
+                push @{$self->{tree}}, [];
+                next;
+            }
         }
 
         # Escaped line ending?
@@ -249,7 +246,7 @@ sub parse {
             # End
             if ($token =~ /^$tag_end$/) {
                 $state = 'text';
-                $multiline_expression = 0;
+                $push_expr = undef;
             }
 
             # Code
@@ -269,11 +266,18 @@ sub parse {
                 # Comments are ignored
                 next if $state eq 'cmnt';
 
-                # Multiline expressions are a bit complicated,
-                # only the first line can be compiled as 'expr'
-                $state = 'code' if $multiline_expression;
-                $multiline_expression = 1
-                    if $state eq 'expr';
+                if ($push_expr) {
+                    $push_expr->($token);
+                    next;
+                }
+
+                $state = 'code' if $push_expr;
+                if ($state eq 'expr') {
+                    my $token = \@token;
+                    $push_expr = sub {
+                        $token->[-1] .= $_[0];
+                    };
+                }
 
                 # Store value
                 push @token, $state, $token;
@@ -546,7 +550,7 @@ Utility function that combines build_mt and call to the generated template build
     # use the result in another template (no double-escapes)
     $enc = render_mt('<h1><?= $_[0] ?></h1>', $hello);
 
-Intertally, the function is equivalent to:
+Internally, the function is equivalent to:
 
     build_mt($template)->(@_);
 
